/*****************************************************************************
*
*  demo.c - CC3000 Main Demo Application
*  Copyright (C) 2011 Texas Instruments Incorporated - http://www.ti.com/
*
*  Redistribution and use in source and binary forms, with or without
*  modification, are permitted provided that the following conditions
*  are met:
*
*    Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
*
*    Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the
*    documentation and/or other materials provided with the   
*    distribution.
*
*    Neither the name of Texas Instruments Incorporated nor the names of
*    its contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
*  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
*  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
*  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
*  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
*  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
*  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
*  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
*  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*****************************************************************************/

#include <msp430.h>
#include "wlan.h" 
#include "evnt_handler.h"    // callback function declaration
#include "nvmem.h"
#include "socket.h"
#include "common.h"
#include "netapp.h"
#include "cc3000.h"
#include "board.h"
#include "string.h"
#include "demo_config.h"
#include "smtp.h"

/** \brief Flag indicating whether user requested to perform FTC */
char runFirstTimeConfig = 0;
unsigned char * ptrFtcAtStartup = (unsigned char *)0x1830;

/** \brief Flag indicating whether to print CC3000 Connection info */
unsigned char obtainIpInfoFlag = FALSE;

void errorHandler();

//*****************************************************************************
//
//!  main
//!
//!  \param  None
//!
//!  \return none
//!
//!  \brief   The main loop is executed here
//
//*****************************************************************************

void main(void)
{
    // Stop WDT  
    WDTCTL = WDTPW + WDTHOLD;
    
    // Start CC3000 State Machine
    resetCC3000StateMachine();
    
    // Initialize Board and CC3000
    initDriver();
    
    // Initialize CC3000 Unsolicited Events Timer
    unsolicicted_events_timer_init();
    
    // Enable interrupts for UART
    __enable_interrupt();
    
    // Main Loop
    while (1)
    {
        // Perform FTC if button pressed in current run or if flag set in FRAM
        // from previous MSP430 Run.
        if(runFirstTimeConfig == 1 || *ptrFtcAtStartup == FIRST_TIME_CONFIG_SET)
        {
            // Clear flag
            *ptrFtcAtStartup = 0xFF;
                
            // Disconnect from AP if we're connected
            wlan_ioctl_set_connection_policy(0,0,0);
            wlan_disconnect();
            
            unsetCC3000MachineState(CC3000_ASSOC);
            
            // Start the First Time Config Process
            StartFirstTimeConfig();
            runFirstTimeConfig = 0;
        }
        
        if(!(currentCC3000State() & CC3000_ASSOC))
        {
            //
            // Check whether First Time Config was run previously. If it was, we
            // use it to connect to an access point. Otherwise, we connect to the
            // default.
            //
           if(!isFTCSet())
			ConnectUsingSSID(DEFAULT_OUT_OF_BOX_SSID);
            
            // Wait until connection is finished
            while (!(currentCC3000State() & CC3000_ASSOC))
            {
                __delay_cycles(100);
                
                //
                // Handle any un-solicited event if required - the function shall be triggered 
                // few times in a second
                // 
                hci_unsolicited_event_handler();
                
                // Check if user pressed button to do First Time Config
                if(runFirstTimeConfig == 1)
                    break;
            }
        }
        
        //
    	// Handle any un-solicited event if required - the function shall be triggered 
    	// few times in a second
    	// 
        hci_unsolicited_event_handler();
        
        // Print out connection status
        if(currentCC3000State() & CC3000_IP_ALLOC && obtainIpInfoFlag == FALSE)
        {            
            unsolicicted_events_timer_disable();
            // Set flag so we don't constantly obtain the IP info
            obtainIpInfoFlag = TRUE;
            turnLedOn(CC3000_IP_ALLOC_IND);
            unsolicicted_events_timer_init();
        }
        
        if(currentCC3000State() & CC3000_IP_ALLOC && obtainIpInfoFlag == TRUE)
        {
            unsolicicted_events_timer_disable();
            
            // Attempt to start data server
            smtpConnect(SMTP_SERVER_NAME, SMTP_SERVER_PORT);
            smtpSend(SRC_EMAIL, "youremail@ti.com", "Your CC3000", "DAJi2839@hotmail.com", "Hello!\n\nThis is your friendly CC3000 Sending an e-mail!\n\n I will now go and start taking over the world.");
			break; // Stop while loop after e-mail is sent
                
            unsolicicted_events_timer_init();
        }
    }
}


#pragma vector=PORT4_VECTOR
__interrupt void Port_4(void)
{    
    switch(__even_in_range(P4IV,P4IV_P4IFG1))
    {        
        case P4IV_P4IFG0:
            StartDebounceTimer();          //
            // disable switch interrupt
            P4IFG &= ~BIT0;                // Clear P4.0 IFG
            P4IE &= ~(BIT0);               // P4.0 interrupt disabled
            P4IFG &= ~BIT0;                // Clear P4.0 IFG
            break;
        
        default:
            break;
    }  
    P4IFG = 0;
}

// Timer A0 interrupt service routine
#pragma vector=TIMER1_A0_VECTOR
__interrupt void TIMER1_A0_ISR(void)
{              
    __no_operation();                          // For debugger
}

// Timer B0 interrupt service routine
#pragma vector = TIMER0_B0_VECTOR
__interrupt void Timer_B (void)
{
    // Check whether button is still pressed. If so, First Time Config
    // should be performed.
    
    if(!(P4IN & BIT0))
    {
        // Button is still pressed, so FTC should be done        
        runFirstTimeConfig = 1;
        
        if(currentCC3000State() & CC3000_SERVER_INIT)
        {
            // Since accept and the server is blocking,
            // we will indicate in non-volatile FRAM that
            // First Time Config should be run at startup.
            *ptrFtcAtStartup = FIRST_TIME_CONFIG_SET;
            restartMSP430();
        }
    }
    
    // Restore Switch Interrupt
    P4IFG &= ~BIT0;                 // Clear P4.0 IFG
    P4IE |= BIT0;                   // P4.0 interrupt enabled
    P4IFG &= ~BIT0;                 // Clear P4.0 IFG
    
    StopDebounceTimer();
}

//*****************************************************************************
//
//!  \brief  Generic Error Handler that blinks LEDs
//!
//!  Look at the Call Stack to see who called the Error handler
//!
//!  \param  None
//!
//!  \return none
//!
//
//*****************************************************************************
void errorHandler()
{
    while(1)
    {
        turnLedOn(CC3000_UNUSED1_IND);
        __delay_cycles(100000);
        toggleLed(CC3000_UNUSED1_IND);
        __delay_cycles(100000);
        __no_operation();
    }
}
